/*************************************************************************
 * ArtisticStylePlugin: Using Artistic Style at Qt Creator.              *
 * Copyright (C) 2010-2011  Lorenz Haas <lykurg@gmail.com>               *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef ARTISTICSTYLECONSTANTS_H
#define ARTISTICSTYLECONSTANTS_H

#include "astyle/astyle.h"

namespace ArtisticStyle
{
  namespace Constants
  {
    // Predefined Style Options
    const char* const ASP_STYLE     = "style";
    const int         ASP_STYLE_DEF = -1;

    // Tab and Bracket Options
    const char* const ASP_INDENT            = "indent";
    const bool        ASP_INDENT_DEF        = true;
    const char* const ASP_INDENTSTYLE       = "indentStyle";
    const char* const ASP_INDENTSTYLE_DEF   = "spaces";
    const char* const ASP_INDENTSIZE        = "indentSize";
    const int         ASP_INDENTSIZE_DEF    = 2;
    const char* const ASP_BRACKETS          = "brackets";
    const bool        ASP_BRACKETS_DEF      = true;
    const char* const ASP_BRACKETSSTYLE     = "bracketsStyle";
    const int         ASP_BRACKETSSTYLE_DEF = astyle::BREAK_MODE;

    // Indentation Options
    const char* const ASP_INDENTCLASSES                = "indentClasses";
    const bool        ASP_INDENTCLASSES_DEF            = true;
    const char* const ASP_INDENTSWITCHES               = "indentSwitches";
    const bool        ASP_INDENTSWITCHES_DEF           = true;
    const char* const ASP_INDENTCASES                  = "indentCases";
    const bool        ASP_INDENTCASES_DEF              = true;
    const char* const ASP_INDENTBRACKETS               = "indentBrackets";
    const bool        ASP_INDENTBRACKETS_DEF           = false;
    const char* const ASP_INDENTBLOCKS                 = "indentBlocks";
    const bool        ASP_INDENTBLOCKS_DEF             = false;
    const char* const ASP_INDENTNAMESPACES             = "indentNamespaces";
    const bool        ASP_INDENTNAMESPACES_DEF         = true;
    const char* const ASP_INDENTLABELS                 = "indentLabels";
    const bool        ASP_INDENTLABELS_DEF             = true;
    const char* const ASP_INDENTPREPROCESSOR           = "indentPreprocessor";
    const bool        ASP_INDENTPREPROCESSOR_DEF       = true;
    const char* const ASP_INDENTCOL1COMMENTS           = "indentCol1Comments";
    const bool        ASP_INDENTCOL1COMMENTS_DEF       = true;
    const char* const ASP_MAXINSTATEMENTINDENT         = "maxInstatementIndent";
    const bool        ASP_MAXINSTATEMENTINDENT_DEF     = true;
    const char* const ASP_MAXINSTATEMENTINDENTSIZE     = "maxInstatementIndentSize";
    const int         ASP_MAXINSTATEMENTINDENTSIZE_DEF = 40;
    const char* const ASP_MINCONDITIONALINDENT         = "minConditionalIndent";
    const bool        ASP_MINCONDITIONALINDENT_DEF     = true;
    const char* const ASP_MINCONDITIONALINDENTSIZE     = "minConditionalIndentSize";
    const int         ASP_MINCONDITIONALINDENTSIZE_DEF = 4;

    // Padding Options
    const char* const ASP_BREAKBLOCKS          = "breakBlocks";
    const bool        ASP_BREAKBLOCKS_DEF      = false;
    const char* const ASP_BREAKBLOCKSALL       = "breakBlocksAll";
    const bool        ASP_BREAKBLOCKSALL_DEF   = false;
    const char* const ASP_PADOPER              = "padOper";
    const bool        ASP_PADOPER_DEF          = true;
    const char* const ASP_PADPAREN             = "padParen";
    const bool        ASP_PADPAREN_DEF         = false;
    const char* const ASP_PADPARENOUT          = "padParenOut";
    const bool        ASP_PADPARENOUT_DEF      = false;
    const char* const ASP_PADPARENIN           = "padParenIn";
    const bool        ASP_PADPARENIN_DEF       = false;
    const char* const ASP_PADHEADER            = "padHeader";
    const bool        ASP_PADHEADER_DEF        = true;
    const char* const ASP_UNPADPAREN           = "unpadParen";
    const bool        ASP_UNPADPAREN_DEF       = true;
    const char* const ASP_DELETEEMPTYLINES     = "deleteEmptyLines";
    const bool        ASP_DELETEEMPTYLINES_DEF = false;
    const char* const ASP_FILLEMPTYLINES       = "fillEmptyLines";
    const bool        ASP_FILLEMPTYLINES_DEF   = false;

    // Formatting Options
    const char* const ASP_BREAKCLOSINGBRACKETS      = "breakClosingBrackets";
    const bool        ASP_BREAKCLOSINGBRACKETS_DEF  = true;
    const char* const ASP_BREAKELSEIFS              = "breakElseifs";
    const bool        ASP_BREAKELSEIFS_DEF          = false;
    const char* const ASP_ADDBRACKETS               = "addBrackets";
    const bool        ASP_ADDBRACKETS_DEF           = false;
    const char* const ASP_ADDONELINEBRACKETS        = "addOneLineBrackets";
    const bool        ASP_ADDONELINEBRACKETS_DEF    = false;
    const char* const ASP_KEEPONELINEBLOCKS         = "keepOneLineBlocks";
    const bool        ASP_KEEPONELINEBLOCKS_DEF     = false;
    const char* const ASP_KEEPONELINESTATEMENTS     = "keepOneLineStatements";
    const bool        ASP_KEEPONELINESTATEMENTS_DEF = false;
    const char* const ASP_CONVERTTABS               = "convertTabs";
    const bool        ASP_CONVERTTABS_DEF           = true;
    const char* const ASP_ALIGNPOINTER              = "alignPointer";
    const bool        ASP_ALIGNPOINTER_DEF          = true;
    const char* const ASP_ALIGNPOINTERSTYLE         = "alignPointerStyle";
    const int         ASP_ALIGNPOINTERSTYLE_DEF     = astyle::ALIGN_TYPE;
  }
}

#endif // ARTISTICSTYLECONSTANTS_H
