/*************************************************************************
 * ArtisticStylePlugin: Using Artistic Style at Qt Creator.              *
 * Copyright (C) 2010-2011  Lorenz Haas <lykurg@gmail.com>               *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#include "artisticstyleoptionpagegui.h"
#include "ui_artisticstyleoptionpagegui.h"

#include "astyle/astyle.h"
#include "artisticstyleconstants.h"

#include <coreplugin/icore.h>

#include <QGridLayout>
#include <QLabel>
#include <QSettings>
#include <QComboBox>
#include <QSpinBox>
#include <QScrollArea>
#include <QGroupBox>

namespace ArtisticStyle
{
  namespace Internal
  {
    ArtisticStyleOptionPageGui::ArtisticStyleOptionPageGui(QWidget* parent) :
      QWidget(parent), ui(new Ui::ArtisticStyleOptionPageGui)
    {
      ui->setupUi(this);
      QSettings* settings = Core::ICore::instance()->settings();
      settings->beginGroup("ArtisticStylePlugin");

      // Predefined Style Options
      ui->predefinedStyle->clear();
      ui->predefinedStyle->addItem("---", -1);
      ui->predefinedStyle->addItem("Allman/Ansi/BSD", astyle::STYLE_ALLMAN);
      ui->predefinedStyle->addItem("Java", astyle::STYLE_JAVA);
      ui->predefinedStyle->addItem("Kernighan & Ritchie ", astyle::STYLE_KandR);
      ui->predefinedStyle->addItem("Stroustrup", astyle::STYLE_STROUSTRUP);
      ui->predefinedStyle->addItem("Whitesmith", astyle::STYLE_WHITESMITH);
      ui->predefinedStyle->addItem("Banner", astyle::STYLE_BANNER);
      ui->predefinedStyle->addItem("GNU", astyle::STYLE_GNU);
      ui->predefinedStyle->addItem("Linux", astyle::STYLE_LINUX);
      ui->predefinedStyle->addItem("Horstmann", astyle::STYLE_HORSTMANN);
      ui->predefinedStyle->addItem("One True Brace Style", astyle::STYLE_1TBS);
      setCurrentForInt(ui->predefinedStyle, settings->value(Constants::ASP_STYLE, Constants::ASP_STYLE_DEF).toInt());

      // Tab and Bracket Options
      ui->indent->setChecked(settings->value(Constants::ASP_INDENT, Constants::ASP_INDENT_DEF).toBool());
      ui->indentStyle->addItem("Spaces", "spaces");
      ui->indentStyle->addItem("Tabulator", "tab");
      ui->indentStyle->addItem("Force Tabulator", "forceTab");
      setCurrentForString(ui->indentStyle, settings->value(Constants::ASP_INDENTSTYLE, Constants::ASP_INDENTSTYLE_DEF).toString());
      ui->indentSize->setValue(settings->value(Constants::ASP_INDENTSIZE, Constants::ASP_INDENTSIZE_DEF).toInt());

      ui->brackets->setChecked(settings->value(Constants::ASP_BRACKETS, Constants::ASP_BRACKETS_DEF).toBool());
      ui->bracketsStyle->addItem("Break", astyle::BREAK_MODE);
      ui->bracketsStyle->addItem("Attach", astyle::ATTACH_MODE);
      ui->bracketsStyle->addItem("Linux ", astyle::LINUX_MODE);
      ui->bracketsStyle->addItem("Stroustrup", astyle::STROUSTRUP_MODE);
      ui->bracketsStyle->addItem("Horstmann", astyle::HORSTMANN_MODE);
      setCurrentForInt(ui->bracketsStyle, settings->value(Constants::ASP_BRACKETSSTYLE, Constants::ASP_BRACKETSSTYLE_DEF).toInt());

      // Indentation Options
      ui->indentClasses->setChecked(settings->value(Constants::ASP_INDENTCLASSES, Constants::ASP_INDENTCLASSES_DEF).toBool());
      ui->indentSwitches->setChecked(settings->value(Constants::ASP_INDENTSWITCHES, Constants::ASP_INDENTSWITCHES_DEF).toBool());
      ui->indentCases->setChecked(settings->value(Constants::ASP_INDENTCASES, Constants::ASP_INDENTCASES_DEF).toBool());
      ui->indentBrackets->setChecked(settings->value(Constants::ASP_INDENTBRACKETS, Constants::ASP_INDENTBRACKETS_DEF).toBool());
      ui->indentBlocks->setChecked(settings->value(Constants::ASP_INDENTBLOCKS, Constants::ASP_INDENTBLOCKS_DEF).toBool());
      ui->indentNamespaces->setChecked(settings->value(Constants::ASP_INDENTNAMESPACES, Constants::ASP_INDENTNAMESPACES_DEF).toBool());
      ui->indentLabels->setChecked(settings->value(Constants::ASP_INDENTLABELS, Constants::ASP_INDENTLABELS_DEF).toBool());
      ui->indentPreprocessor->setChecked(settings->value(Constants::ASP_INDENTPREPROCESSOR, Constants::ASP_INDENTPREPROCESSOR_DEF).toBool());
      ui->indentCol1Comments->setChecked(settings->value(Constants::ASP_INDENTCOL1COMMENTS, Constants::ASP_INDENTCOL1COMMENTS_DEF).toBool());
      ui->maxInstatementIndent->setChecked(settings->value(Constants::ASP_MAXINSTATEMENTINDENT, Constants::ASP_MAXINSTATEMENTINDENT_DEF).toBool());
      ui->maxInstatementIndentSize->setValue(settings->value(Constants::ASP_MAXINSTATEMENTINDENTSIZE, Constants::ASP_MAXINSTATEMENTINDENTSIZE_DEF).toInt());
      ui->minConditionalIndent->setChecked(settings->value(Constants::ASP_MINCONDITIONALINDENT, Constants::ASP_MINCONDITIONALINDENT_DEF).toBool());
      ui->minConditionalIndentSize->setValue(settings->value(Constants::ASP_MINCONDITIONALINDENTSIZE, Constants::ASP_MINCONDITIONALINDENTSIZE_DEF).toInt());

      // Padding Options
      ui->breakBlocks->setChecked(settings->value(Constants::ASP_BREAKBLOCKS, Constants::ASP_BREAKBLOCKS_DEF).toBool());
      ui->breakBlocksAll->setChecked(settings->value(Constants::ASP_BREAKBLOCKSALL, Constants::ASP_BREAKBLOCKSALL_DEF).toBool());
      ui->padOper->setChecked(settings->value(Constants::ASP_PADOPER, Constants::ASP_PADOPER_DEF).toBool());
      ui->padParen->setChecked(settings->value(Constants::ASP_PADPAREN, Constants::ASP_PADPAREN_DEF).toBool());
      ui->padParenOut->setChecked(settings->value(Constants::ASP_PADPARENOUT, Constants::ASP_PADPARENOUT_DEF).toBool());
      ui->padParenIn->setChecked(settings->value(Constants::ASP_PADPARENIN, Constants::ASP_PADPARENIN_DEF).toBool());
      ui->padHeader->setChecked(settings->value(Constants::ASP_PADHEADER, Constants::ASP_PADHEADER_DEF).toBool());
      ui->unpadParen->setChecked(settings->value(Constants::ASP_UNPADPAREN, Constants::ASP_UNPADPAREN_DEF).toBool());
      ui->deleteEmptyLines->setChecked(settings->value(Constants::ASP_DELETEEMPTYLINES, Constants::ASP_DELETEEMPTYLINES_DEF).toBool());
      ui->fillEmptyLines->setChecked(settings->value(Constants::ASP_FILLEMPTYLINES, Constants::ASP_FILLEMPTYLINES).toBool());

      // Formatting Options
      ui->breakClosingBrackets->setChecked(settings->value(Constants::ASP_BREAKCLOSINGBRACKETS, Constants::ASP_BREAKCLOSINGBRACKETS_DEF).toBool());
      ui->breakElseifs->setChecked(settings->value(Constants::ASP_BREAKELSEIFS, Constants::ASP_BREAKELSEIFS_DEF).toBool());
      ui->addBrackets->setChecked(settings->value(Constants::ASP_ADDBRACKETS, Constants::ASP_ADDBRACKETS_DEF).toBool());
      ui->addOneLineBrackets->setChecked(settings->value(Constants::ASP_ADDONELINEBRACKETS, Constants::ASP_ADDONELINEBRACKETS_DEF).toBool());
      ui->keepOneLineBlocks->setChecked(settings->value(Constants::ASP_KEEPONELINEBLOCKS, Constants::ASP_KEEPONELINEBLOCKS_DEF).toBool());
      ui->keepOneLineStatements->setChecked(settings->value(Constants::ASP_KEEPONELINESTATEMENTS, Constants::ASP_KEEPONELINESTATEMENTS_DEF).toBool());
      ui->convertTabs->setChecked(settings->value(Constants::ASP_CONVERTTABS, Constants::ASP_CONVERTTABS).toBool());
      ui->alignPointer->setChecked(settings->value(Constants::ASP_ALIGNPOINTER, Constants::ASP_ALIGNPOINTER_DEF).toBool());
      ui->alignPointerStyle->clear();
      ui->alignPointerStyle->addItem("Type", astyle::ALIGN_TYPE);
      ui->alignPointerStyle->addItem("Middle", astyle::ALIGN_MIDDLE);
      ui->alignPointerStyle->addItem("Name ", astyle::ALIGN_NAME);
      setCurrentForInt(ui->alignPointerStyle, settings->value(Constants::ASP_ALIGNPOINTERSTYLE, Constants::ASP_ALIGNPOINTERSTYLE_DEF).toInt());

      settings->endGroup();
    }

    ArtisticStyleOptionPageGui::~ArtisticStyleOptionPageGui()
    {
      delete ui;
    }

    inline void ArtisticStyleOptionPageGui::setCurrentForInt(QComboBox* box, int value)
    {
      for (int i = 0; i < box->count(); ++i)
      {
        if (box->itemData(i).toInt() == value)
        {
          box->setCurrentIndex(i);
          break;
        }
      }
    }

    inline void ArtisticStyleOptionPageGui::setCurrentForString(QComboBox* box, const QString& value)
    {
      for (int i = 0; i < box->count(); ++i)
      {
        if (box->itemData(i).toString() == value)
        {
          box->setCurrentIndex(i);
          break;
        }
      }
    }

    inline bool ArtisticStyleOptionPageGui::setValueWithCheck(QSettings* settings, const QString& key, const QVariant& value)
    {
      if (settings->value(key) == value)
        return false;

      settings->setValue(key, value);
      return true;
    }

    bool ArtisticStyleOptionPageGui::save()
    {
      bool changed = false;
      QSettings* settings = Core::ICore::instance()->settings();
      settings->beginGroup("ArtisticStylePlugin");

      // Predefined Style Options
      changed |= setValueWithCheck(settings, Constants::ASP_STYLE, ui->predefinedStyle->itemData(ui->predefinedStyle->currentIndex()));

      // Tab and Bracket Options
      changed |= setValueWithCheck(settings, Constants::ASP_INDENT, ui->indent->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTSTYLE, ui->indentStyle->itemData(ui->indentStyle->currentIndex()));
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTSIZE, ui->indentSize->value());
      changed |= setValueWithCheck(settings, Constants::ASP_BRACKETS, ui->brackets->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_BRACKETSSTYLE, ui->bracketsStyle->itemData(ui->bracketsStyle->currentIndex()));

      // Indentation Options
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTCLASSES, ui->indentClasses->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTSWITCHES, ui->indentSwitches->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTCASES, ui->indentCases->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTBRACKETS, ui->indentBrackets->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTBLOCKS, ui->indentBlocks->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTNAMESPACES, ui->indentNamespaces->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTLABELS, ui->indentLabels->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTPREPROCESSOR, ui->indentPreprocessor->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_INDENTCOL1COMMENTS, ui->indentCol1Comments->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_MAXINSTATEMENTINDENT, ui->maxInstatementIndent->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_MAXINSTATEMENTINDENTSIZE, ui->maxInstatementIndentSize->value());
      changed |= setValueWithCheck(settings, Constants::ASP_MINCONDITIONALINDENT, ui->minConditionalIndent->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_MINCONDITIONALINDENTSIZE, ui->minConditionalIndentSize->value());

      // Padding Options
      changed |= setValueWithCheck(settings, Constants::ASP_BREAKBLOCKS, ui->breakBlocks->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_BREAKBLOCKSALL, ui->breakBlocksAll->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_PADOPER, ui->padOper->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_PADPAREN, ui->padParen->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_PADPARENOUT, ui->padParenOut->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_PADPARENIN, ui->padParenIn->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_PADHEADER, ui->padHeader->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_UNPADPAREN, ui->unpadParen->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_DELETEEMPTYLINES, ui->deleteEmptyLines->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_FILLEMPTYLINES, ui->fillEmptyLines->isChecked());

      // Formatting Options
      changed |= setValueWithCheck(settings, Constants::ASP_BREAKCLOSINGBRACKETS, ui->breakClosingBrackets->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_BREAKELSEIFS, ui->breakElseifs->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_ADDBRACKETS, ui->addBrackets->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_ADDONELINEBRACKETS, ui->addOneLineBrackets->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_KEEPONELINEBLOCKS, ui->keepOneLineBlocks->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_KEEPONELINESTATEMENTS, ui->keepOneLineStatements->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_CONVERTTABS, ui->convertTabs->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_ALIGNPOINTER, ui->alignPointer->isChecked());
      changed |= setValueWithCheck(settings, Constants::ASP_ALIGNPOINTERSTYLE, ui->alignPointerStyle->itemData(ui->alignPointerStyle->currentIndex()));

      settings->endGroup();
      return changed;
    }
  } // namespace Internal
} // namespace ArtisticStyle
